/*
 * Copyright (c) 2019 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <stdio.h>
#include <sqlite3.h>

#include "alarm-manager-db.h"

extern sqlite3 *alarmmgr_db;

int migration_func_version_1(void)
{
	static const char *query = "CREATE TABLE IF NOT EXISTS alarmmgr_temp (\n" \
		"	alarm_id INTEGER PRIMARY KEY, \n" \
		"	start INTEGER, \n" \
		"	end INTEGER, \n" \
		"	uid INTEGER, \n" \
		"	global INTEGER, \n" \
		"	is_disabled INTEGER, \n" \
		"	caller_pkgid TEXT, \n" \
		"	callee_pkgid TEXT, \n" \
		"	app_unique_name TEXT, \n" \
		"	app_service_name TEXT, \n" \
		"	app_service_name_mod TEXT, \n" \
		"	bundle TEXT, \n" \
		"	noti_len INTEGER, \n" \
		"	noti TEXT, \n" \
		"	year INTEGER, \n" \
		"	month INTEGER, \n" \
		"	day INTEGER, \n" \
		"	hour INTEGER, \n" \
		"	min INTEGER, \n" \
		"	sec INTEGER, \n" \
		"	msec INTEGER, \n" \
		"	day_of_week INTEGER, \n" \
		"	repeat INTEGER, \n" \
		"	alarm_type INTEGER, \n" \
		"	reserved_info INTEGER, \n" \
		"	dst_service_name TEXT, \n" \
		"	dst_service_name_mod TEXT \n" \
		")";

	static const char *insert_query = "INSERT INTO alarmmgr_temp (alarm_id, start, \
		end, uid, global, is_disabled, caller_pkgid, callee_pkgid, \
		app_unique_name, app_service_name, app_service_name_mod, \
		bundle, noti_len, noti, year, month, day, hour, min, sec, msec,\
		day_of_week, repeat, alarm_type, reserved_info, \
		dst_service_name, dst_service_name_mod) \
		SELECT alarm_id, start, end, uid, global, is_disabled, \
		caller_pkgid, callee_pkgid, app_unique_name, app_service_name, \
		app_service_name_mod, bundle, noti_len, noti, year, month, \
		day, hour, min, sec, msec, day_of_week, repeat, alarm_type, \
		reserved_info, dst_service_name, dst_service_name_mod \
		FROM alarmmgr";

	char *error_message = NULL;
	int ret;

	LOGD("DB migration init version start");

	ret = sqlite3_exec(alarmmgr_db, "DROP TABLE IF EXISTS alarmmgr_temp", NULL,
			NULL, &error_message);
	if (ret != SQLITE_OK) {
		LOGE("Don't execute drop, error message = %s", error_message);
		sqlite3_free(error_message);
		return ret;
	}

	ret = sqlite3_exec(alarmmgr_db, query, NULL, NULL, &error_message);
	if (ret != SQLITE_OK) {
		LOGE("Don't execute query = %s, error message = %s", query, error_message);
		sqlite3_free(error_message);
		return ret;
	}

	ret = sqlite3_exec(alarmmgr_db, insert_query, NULL, NULL, &error_message);
	if (ret != SQLITE_OK) {
		LOGE("Don't execute query = %s, error message = %s", insert_query, error_message);
		sqlite3_free(error_message);
		return ret;
	}

	ret = sqlite3_exec(alarmmgr_db, "DROP TABLE alarmmgr", NULL, NULL, &error_message);
	if (ret != SQLITE_OK) {
		LOGE("Don't execute drop alarmmgr, error message = %s", error_message);
		sqlite3_free(error_message);
		return ret;
	}

	ret = sqlite3_exec(alarmmgr_db, "ALTER TABLE alarmmgr_temp RENAME TO alarmmgr",
			NULL, NULL, &error_message);
	if (ret != SQLITE_OK) {
		LOGE("Don't execute alter, error message = %s", error_message);
		sqlite3_free(error_message);
		return ret;
	}

	return ALARMMGR_RESULT_SUCCESS;
}
