/*
_all * Copyright (c) 2019 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#include <gmock/gmock.h>
#include <gtest/gtest.h>
#include <unistd.h>
#include <stdio.h>

#include <appsvc.h>

#include "unittest.h"
#include "alarm.h"
#include "alarm-internal.h"
#include "mock/gio_fake.h"
#include "mock/glib_fake.h"
#include "mock/aul_fake.h"
#include "mock/notification_fake.h"
#include "mock/pkgmgr_info_fake.h"

class AlarmServerTest : public ::testing::Test {
  protected:
    void SetUp() override {
    }

    void TearDown() override {
    }
};

int __aul_app_get_pkgid_bypid_for_uid_fake(int pid, char* pkgid, int size,
    uid_t uid)
{
  return 0;
}

static GDBusConnection* __g_bus_get_sync_fake(GBusType type, GCancellable *cancel,
    GError **error)
{
  GDBusConnection *con = (GDBusConnection*)g_object_new(G_TYPE_OBJECT, NULL);
  return con;
}

static guint __g_dbus_connection_signal_subscribe_fake(GDBusConnection *con, const gchar* sender,
    const gchar* interface, const gchar* member, const gchar* object,
    const gchar* arg, GDBusSignalFlags flags, GDBusSignalCallback callback,
    gpointer user_data, GDestroyNotify noti)
{
  return 1;
}

static GDBusNodeInfo* __g_dbus_node_info_new_for_xml_fake(const gchar* xml, GError **error)
{
  GDBusNodeInfo *info = (GDBusNodeInfo*)g_object_new(G_TYPE_OBJECT, NULL);
  return info;
}

static guint __g_bus_own_name_on_connection_fake(GDBusConnection *con, const gchar* name,
    GBusNameOwnerFlags flags, GBusNameAcquiredCallback bus_name_acquired,
    GBusNameLostCallback lost, gpointer user_data, GDestroyNotify noti)
{
  return 1;
}

gpointer __g_hash_table_lookup_fake(GHashTable *table, gconstpointer p)
{
  return NULL;
}

int __pkgmgrinfo_pkginfo_get_usr_pkginfo_fake(const char *pkgid, uid_t uid,
    pkgmgrinfo_pkginfo_h *handle)
{
  return 0;
}

int __pkgmgrinfo_pkginfo_get_api_version_fake(
    pkgmgrinfo_pkginfo_h handle, char **version)
{
  return 0;
}

int __pkgmgrinfo_pkginfo_get_is_global_fake(
    pkgmgrinfo_pkginfo_h handle, bool *is_global)
{
  *is_global = true;
  return 0;
}

int __pkgmgrinfo_appinfo_get_usr_appinfo_fake(const char *pkgid, uid_t uid,
    pkgmgrinfo_pkginfo_h *handle)
{
  return 0;
}

int __pkgmgrinfo_appinfo_get_component_type_fake(pkgmgrinfo_appinfo_h handle, char **type)
{
  return 0;
}

int __pkgmgrinfo_pkginfo_destroy_fake(pkgmgrinfo_pkginfo_h handle)
{
  return 0;
}

int __pkgmgrinfo_appinfo_destroy_fake(pkgmgrinfo_appinfo_h handle)
{
  return 0;
}

bundle *__bundle_decode_fake(const bundle_raw* raw, const int len)
{
  bundle *b;

  b = bundle_create();
  bundle_add_str(b, "__APP_SVC_PKG_NAME__", "org.tizen.alarm");

  return b;
}

TEST_F(AlarmServerTest, alarm_manager_alarm_create_p)
{
  int ret;
  int alarm_id;
  GVariant *param = NULL;
  gint64 reserved = 1;

  g_hash_table_lookup_fake.custom_fake = __g_hash_table_lookup_fake;
  g_bus_get_sync_fake.custom_fake = __g_bus_get_sync_fake;
  g_dbus_connection_signal_subscribe_fake.custom_fake = __g_dbus_connection_signal_subscribe_fake;
  g_dbus_node_info_new_for_xml_fake.custom_fake = __g_dbus_node_info_new_for_xml_fake;
  g_bus_own_name_on_connection_fake.custom_fake = __g_bus_own_name_on_connection_fake;

  _alarm_initialize();
  param = g_variant_new("(ssiiiiiiiiiiiiixss)",
      "app_service_name", "app_service_name_mod",
      2019, 11, 1, 12, 59, 59, 400, 2020, 12, 29, 3, 1, 1, reserved,
      "reserved_service_name", "reserved_service_name_mod");

  ret = alarm_manager_alarm_create(param, 5001, 1, &alarm_id);
  EXPECT_EQ(ALARMMGR_RESULT_SUCCESS, ret);
}

TEST_F(AlarmServerTest, alarm_manager_alarm_create_appsvc_p)
{
  int ret;
  int alarm_id;
  GVariant *param = NULL;
  bundle *b = NULL;
  bundle_raw *b_data = NULL;
  int len = 0;
  gint64 reserved = 1;

  aul_app_get_pkgid_bypid_for_uid_fake.custom_fake = __aul_app_get_pkgid_bypid_for_uid_fake;
  g_hash_table_lookup_fake.custom_fake = __g_hash_table_lookup_fake;

  pkgmgrinfo_pkginfo_get_usr_pkginfo_fake.custom_fake =
    __pkgmgrinfo_pkginfo_get_usr_pkginfo_fake;
  pkgmgrinfo_pkginfo_get_api_version_fake.custom_fake =
    __pkgmgrinfo_pkginfo_get_api_version_fake;

  pkgmgrinfo_appinfo_get_usr_appinfo_fake.custom_fake =
    __pkgmgrinfo_appinfo_get_usr_appinfo_fake;

  pkgmgrinfo_appinfo_get_component_type_fake.custom_fake =
    __pkgmgrinfo_appinfo_get_component_type_fake;
  pkgmgrinfo_appinfo_destroy_appinfo_fake.custom_fake =
    __pkgmgrinfo_appinfo_destroy_fake;

  //bundle_decode_fake.custom_fake = __bundle_decode_fake;

  b = bundle_create();
  bundle_add_str(b, "__APP_SVC_PKG_NAME__", "org.tizen.alarm");
  bundle_encode(b, &b_data, &len);

  param = g_variant_new("(iiiiiiiiiixiixs)",
      2019, 11, 1, 12, 59, 59,
      2020, 12, 29,
      1, reserved, 1, 0, reserved,
      (char *)b_data);
  ret = alarm_manager_alarm_create_appsvc(param, 5001, 1, &alarm_id);
  bundle_free(b);
  if (b_data)
    free(b_data);
  EXPECT_EQ(ALARMMGR_RESULT_SUCCESS, ret);
}

TEST_F(AlarmServerTest, alarm_manager_alarm_create_periodic_p)
{
  int ret;
  int alarm_id;
  GVariant *param = NULL;

  param = g_variant_new("(ssiii)",
      "app_service_name", "app_service_name_mod", 1, 0, 1);

  ret = alarm_manager_alarm_create_periodic(param, 5001, 1, &alarm_id);
  EXPECT_EQ(ALARMMGR_RESULT_SUCCESS, ret);
}

TEST_F(AlarmServerTest, alarm_manager_alarm_create_noti_p)
{
  int ret;
  int alarm_id;
  GVariant *param = NULL;
  gint64 reserved = 1;

  param = g_variant_new("(iiiiiiiiiixiixs)",
      2019, 11, 1, 12, 59, 59,
      2020, 12, 29, 3,
      0, reserved, 1, 0, reserved,
      "bundle");


  ret = alarm_manager_alarm_create_noti(param, 5001, 1, &alarm_id);
  EXPECT_EQ(ALARMMGR_RESULT_SUCCESS, ret);
}

TEST_F(AlarmServerTest, alarm_manager_alarm_delete_p)
{
  int ret;
  GVariant *param = NULL;

  param = g_variant_new("(i)", 1);

  ret = alarm_manager_alarm_delete(param, 5001, 1);
  EXPECT_EQ(ERR_ALARM_INVALID_ID, ret);
}

TEST_F(AlarmServerTest, alarm_manager_alarm_delete_all_p)
{
  int ret;
  GVariant *param = NULL;

  ret = alarm_manager_alarm_delete_all(param, 5001, 1);
  EXPECT_EQ(ALARMMGR_RESULT_SUCCESS, ret);
}

TEST_F(AlarmServerTest, alarm_manager_alarm_update_n)
{
  int ret;
  int alarm_id = 1;
  GVariant *param = NULL;
  gint64 reserved = 1;

  param = g_variant_new("(iiiiiiiiiixiixi)",
      alarm_id,
      2019, 11, 1, 12, 59, 59,
      2020, 12, 29, 0,
      reserved, 0, 0, reserved, 0);

  ret = alarm_manager_alarm_update(param, 5001, 1);
  EXPECT_EQ(ERR_ALARM_INVALID_ID, ret);
}

TEST_F(AlarmServerTest, alarm_manager_alarm_get_number_of_ids_p)
{
  int ret;
  int num_of_ids;

  ret = alarm_manager_alarm_get_number_of_ids(5001, 1, &num_of_ids);
  EXPECT_EQ(ALARMMGR_RESULT_SUCCESS, ret);
}

TEST_F(AlarmServerTest, alarm_manager_alarm_get_list_of_ids_p)
{
  int ret;
  int num_of_alarm;
  GVariant *alarm_array = NULL;
  GVariant *param = NULL;

  param = g_variant_new("(i)", 10);

  ret = alarm_manager_alarm_get_list_of_ids(param, 5001, 1, &alarm_array,
      &num_of_alarm);
  EXPECT_EQ(ALARMMGR_RESULT_SUCCESS, ret);
}

TEST_F(AlarmServerTest, alarm_manager_alarm_get_appsvc_info_n)
{
  int ret;
  gchar *b_data = NULL;
  GVariant *param = NULL;

  param = g_variant_new("(i)", 10);

  ret = alarm_manager_alarm_get_appsvc_info(param, 5001, &b_data);
  EXPECT_EQ(ERR_ALARM_INVALID_ID, ret);
}

TEST_F(AlarmServerTest, alarm_manager_alarm_get_noti_info_n)
{
  int ret;
  gchar *b_data = NULL;
  GVariant *param = NULL;

  param = g_variant_new("(i)", 10);

  ret = alarm_manager_alarm_get_noti_info(param, 5001, &b_data);
  EXPECT_EQ(ERR_ALARM_INVALID_ID, ret);
}

TEST_F(AlarmServerTest, alarm_manager_alarm_get_info_n)
{
  int ret;
  base_info_t info;
  GVariant *param = NULL;

  param = g_variant_new("(i)", 10);

  ret = alarm_manager_alarm_get_info(param, 5001, &info);
  EXPECT_EQ(ERR_ALARM_INVALID_ID, ret);
}

TEST_F(AlarmServerTest, alarm_manager_alarm_get_next_duetime_n)
{
  int ret;
  time_t duetime;
  GVariant *param = NULL;

  param = g_variant_new("(i)", 10);

  ret = alarm_manager_alarm_get_next_duetime(param, 5001, &duetime);
  EXPECT_EQ(ERR_ALARM_INVALID_ID, ret);
}

TEST_F(AlarmServerTest, alarm_manager_alarm_get_all_info_p)
{
  int ret;
  char *db_path;

  ret = alarm_manager_alarm_get_all_info(5001, &db_path);
  EXPECT_EQ(ALARMMGR_RESULT_SUCCESS, ret);
}

TEST_F(AlarmServerTest, alarm_manager_alarm_set_rtc_time_n)
{
  int ret;
  GVariant *param = NULL;

  param = g_variant_new("(iiiiii)", 2019, 11, 1, 12, 12, 12);

  ret = alarm_manager_alarm_set_rtc_time(param);
  EXPECT_EQ(ERR_ALARM_SYSTEM_FAIL, ret);
}

TEST_F(AlarmServerTest, alarm_manager_alarm_set_time_n)
{
  int ret;
  GVariant *param = NULL;

  param = g_variant_new("(x)", 20190012);

  ret = alarm_manager_alarm_set_time(param, 1);
  EXPECT_EQ(ERR_ALARM_SYSTEM_FAIL, ret);
}

TEST_F(AlarmServerTest, alarm_manager_alarm_set_time_with_propagation_delay_n)
{
  int ret;
  GVariant *param = NULL;

  param = g_variant_new("(xxxx)", 20190012, 2949294, 292929, 29299292);

  ret = alarm_manager_alarm_set_time_with_propagation_delay(param, 1);
  EXPECT_EQ(ERR_ALARM_SYSTEM_FAIL, ret);
}

TEST_F(AlarmServerTest, alarm_manager_alarm_set_timezone_n)
{
  int ret;
  GVariant *param = NULL;

  param = g_variant_new("(s)", "/usr/share/zoneinfo/Asia/Seoul");

  ret = alarm_manager_alarm_set_timezone(param);
  EXPECT_EQ(ERR_ALARM_SYSTEM_FAIL, ret);
}

TEST_F(AlarmServerTest, alarm_manager_alarm_set_global_n)
{
  int ret;
  GVariant *param = NULL;
  GVariant *param2 = NULL;
  gint64 reserved = 1;
  int alarm_id;

  g_hash_table_lookup_fake.custom_fake = __g_hash_table_lookup_fake;
  pkgmgrinfo_pkginfo_get_usr_pkginfo_fake.custom_fake =
    __pkgmgrinfo_pkginfo_get_usr_pkginfo_fake;
  pkgmgrinfo_pkginfo_is_global_fake.custom_fake =
    __pkgmgrinfo_pkginfo_get_is_global_fake;
  pkgmgrinfo_pkginfo_destroy_pkginfo_fake.custom_fake =
    __pkgmgrinfo_pkginfo_destroy_fake;

  param = g_variant_new("(ssiiiiiiiiiiiiixss)",
      "app_service_name", "app_service_name_mod",
      2019, 11, 1, 12, 59, 59, 400, 2020, 12, 29, 3, 1, 1, reserved,
      "reserved_service_name", "reserved_service_name_mod");

  ret = alarm_manager_alarm_create(param, 5001, 1, &alarm_id);
  param2 = g_variant_new("(ib)", alarm_id, true);

  ret = alarm_manager_alarm_set_global(param2, 5001);
  EXPECT_EQ(ERR_ALARM_SYSTEM_FAIL, ret);
}

TEST_F(AlarmServerTest, alarm_manager_alarm_get_global_n)
{
  int ret;
  GVariant *param = NULL;
  gboolean global;

  param = g_variant_new("(i)", 1);

  ret = alarm_manager_alarm_get_global(param, &global);
  EXPECT_EQ(ERR_ALARM_INVALID_ID, ret);
}
